using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace HelloXNA
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Model myModel;
        float aspectRatio;

        Vector3 modelPosition = Vector3.Zero;
        float modelRotation = 0.0f;

        Vector3 cameraPosition = new Vector3(0.0f, 2.0f, 5.0f);

        bool specularReduced = true;
        KeyboardState kbdState;

        Texture2D[] texs = new Texture2D[6];

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            kbdState = Keyboard.GetState();

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            myModel = Content.Load<Model>("cube");
            aspectRatio = graphics.GraphicsDevice.Viewport.AspectRatio;

            // Load textures
            texs[0] = Content.Load<Texture2D>("CGSphere0");
            texs[1] = Content.Load<Texture2D>("CGSphere1");
            texs[2] = Content.Load<Texture2D>("CGSphere2");
            texs[3] = Content.Load<Texture2D>("CGSphere3");
            texs[4] = Content.Load<Texture2D>("CGSphere4");
            texs[5] = Content.Load<Texture2D>("Basics");

            InitEffects();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed ||
                Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();
            if (Keyboard.GetState().IsKeyDown(Keys.S) && !kbdState.IsKeyDown(Keys.S))
                specularReduced = !specularReduced;

            kbdState = Keyboard.GetState();

            modelRotation += (float)gameTime.ElapsedGameTime.TotalMilliseconds * MathHelper.ToRadians(0.01f);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            LazyDrawModel(myModel, Matrix.CreateRotationY(modelRotation) * Matrix.CreateRotationZ(modelRotation * 3));

            base.Draw(gameTime);
        }

        private void LazyDrawModel(Model model, Matrix world)
        {
            Matrix[] transforms = new Matrix[model.Bones.Count];
            model.CopyAbsoluteBoneTransformsTo(transforms);

            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.World = transforms[mesh.ParentBone.Index] * world;
                }
                mesh.Draw();
            }
        }

        private void InitEffects()
        {
            int idx = 0;

            foreach (ModelMesh mesh in myModel.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.Texture = texs[idx++];
                    effect.EnableDefaultLighting();
                    if (specularReduced)
                    {
                        effect.DirectionalLight0.SpecularColor *= 0.25f;
                        effect.DirectionalLight1.SpecularColor *= 0.25f;
                        effect.DirectionalLight2.SpecularColor *= 0.25f;
                    }
                    effect.View = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
                    effect.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 10000.0f);
                }
            }
        }
    }
}
